/*
Copyright (C) 2017, Vershinin Ivan
*/

package ru.vershinin.medical.btdog;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import java.util.BitSet;

public class ImageSender {
    private Bitmap image;
    public int imgHeight, imgWidth; // размеры исходной картинки
    public int prtHeight, prtWidth; // размеры текущей части картинки

    // создаем коллбек и его метод
    interface Callback {
        void sendPartialImage();
    }

    Callback callback;

    public void registerCallBack(Callback callback){
        this.callback = callback;
    }

    private final Context context;

    public ImageSender(Context _context) {
        context = _context;
    }

    // Преобразует значение BitSet в int
    public static int bitSetToInt(BitSet bits) {
        int value = 0;
        for (int i = 0; i < bits.length(); ++i) {
            value += bits.get(i) ? (1 << i) : 0;
        }
        return value;
    }

    // Возвращает true если цвет черный
    public boolean isBlack(int color) {
        int blue = color & 0xff;
        int green = (color & 0xff00) >> 8;
        int red = (color & 0xff0000) >> 16;

        if ((blue < 200) && (green < 200) && (red < 200)) {
            return true;
        } else
            return false;
    }

    public void setImageCoord(int value, byte[] msg, boolean xcoord) {
        StringBuilder sb = new StringBuilder();
        sb.append(String.format("%02X", value));
        if (xcoord) { // x
            msg[3] = (byte)sb.charAt(0);
            msg[4] = (byte)sb.charAt(1);
        } else // y
        {
            msg[5] = (byte)sb.charAt(0);
            msg[6] = (byte)sb.charAt(1);
        }
    }

    public void setImageSize(int w, int h, byte[] pkg) {
        StringBuilder sb = new StringBuilder();
        // Width
        sb.append(String.format("%01X", w));
        pkg[7] = (byte)sb.charAt(0);
        // Height
        sb.delete(0, sb.length());
        sb.append(String.format("%01X", h));
        pkg[8] = (byte)sb.charAt(0);
    }

    public void setPacket(int x, int y, int w, int h, byte[] pkg) {
        setImageCoord(x, pkg, true);
        setImageCoord(y, pkg, false);
        setImageSize(w, h, pkg);
        // вызываем метод обратного вызова
        callback.sendPartialImage();
        // очистить область координат, размера и данных массива
        for (int i = 3; i < Constants.IMG_PACKET_SIZE; i++) {
            pkg[i] = 0;
        }
    }

    public boolean Prepare(int imageID, int AX0, int AY0) {
        boolean Result = false;
        image = BitmapFactory.decodeResource(context.getResources(), imageID);
        imgHeight = image.getHeight();
        imgWidth = image.getWidth();
        if (((AX0 + imgWidth) > Constants.LCD_WIDTH) || ((AY0 + imgHeight) > Constants.LCD_HEIGHT))
        {
            return Result;
        }
        Result = true;
        return Result;
    }

    public boolean Go(int AX0, int AY0, byte[] APacket) {
        boolean Result = false;
        int dx, dy, y, c, b, bc;
        BitSet bs = new BitSet();

        for (dx = 0; dx < imgWidth; dx += 8) {
            dy = 0;
            prtHeight = 8;
            prtWidth = 8;
            bc = 0;
            for (y = 0; y < imgHeight; y++) {
                bs.clear();
                for (c = 0; c < 8; c++) {
                    if ((dx + c) >= imgWidth) {
                        prtWidth = c;
                        break;
                    }
                    if (isBlack(image.getPixel(dx + c, y))) { // черный?
                        bs.set(7 - c);
                    }
                }
                // очередной байт готов
                b = bitSetToInt(bs);
                APacket[9 + bc] = (byte)b;
                bc++;
                // набралость 8 байт?
                if ((y % 8) == 7) {
                    setPacket(AX0 + dx, AY0 + dy, prtWidth, prtHeight, APacket);
                    dy += 8;
                    bc = 0;
                }
            }
            // если высота картинки не кратна 8, то вывести остаток
            if ((y % 8) != 0) {
                prtHeight = y % 8;
                setPacket(AX0 + dx, AY0 + dy, prtWidth, prtHeight, APacket);
            }
        }
        Result = true;
        return Result;
    }
}
