/*
Copyright (C) 2017, Vershinin Ivan
*/

package ru.vershinin.medical.btdog;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Handler;
import android.text.Html;
import android.util.Log;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import java.lang.ref.WeakReference;
import java.nio.charset.Charset;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.lang.Math;
import java.util.Date;
import java.util.Locale;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

public class MainActivity extends Activity implements ImageSender.Callback {

    public static final String TAG = "MAIN_ACTIVITY";

    // Код запроса диалога (намерения) включения Bluetooth
    private static final int REQUEST_ENABLE_BT = 3;

    // Визуальные элементы окна
    private Button btnRun;
    private EditText etTextForSend;
    private EditText etCurPosX;
    private EditText etCurPosY;
    private EditText etTextSize;
    private CheckBox chbWrap;
    private RadioGroup rgTextColor;
    private RadioGroup rgColorMode;
    private EditText etX1;
    private EditText etY1;
    private EditText etX2;
    private EditText etY2;
    private EditText etContrast;
    private CheckBox chbInvertDisplay;
    private TextView tvX2, tvY2, tvY1, tvX1;
    private TextView tvTextColor;
    private TextView tvDrawModeCap;
    private CheckBox chbFillMode;
    private EditText etRotate;
    private ImageView ivImage;
    private EditText etImgX;
    private EditText etImgY;

    // Имя подключенного устройства
    private String deviceName = null;
    // MAC-адрес Bluetooth модуля ("00:15:83:33:42:66")
    static String deviceMacAddr;
    // Флаг "прибор найден"
    private boolean deviceIsFound = false;
    // Bluetooth адаптер своего телефона
    private BluetoothAdapter btAdapter = null;
    // Служба Bluetooth-соединения
    private BluetoothService btService = null;
    // Прогресс-диалог при обнаружении Bluetooth-устройств
    private ProgressDialog progressDialog;
    // Флаг "вид подключения"
    private boolean secureConnect;
    // Флаг "подключено"
    private boolean btConnected = false;
    // Переменные для настроек цвета
    private int textColor = 1;
    private int textBgColor = 0;
    private boolean textColorMode = true;
    private int drawMode = 1; // Dot, Line, Rectangle or Circle
    // Переменные для картинки
    private String[] imgNames;
    private int imageIndex = 0;
    private int imageID;
    private byte[] imagePacket = new byte[Constants.IMG_PACKET_SIZE];
    private ImageSender imgSender;
    // Переменные для вывода часов
    private boolean analogTime = false;
    private boolean sendTime = false;
    private Timer secTimer;
    private TimerTask secTask;

    SimpleDateFormat sdf_time = new SimpleDateFormat("HH:mm", Locale.getDefault());
    SimpleDateFormat sdf_secs = new SimpleDateFormat("ss", Locale.getDefault());
    SimpleDateFormat sdf_date = new SimpleDateFormat("d MMM", Locale.getDefault());
    SimpleDateFormat sdf_day = new SimpleDateFormat("E", Locale.getDefault());

    class secTimerTask extends TimerTask {
        private Calendar calendar;
        private Date curDate;
        private int m, h;
        private StringBuilder tmp = new StringBuilder();
        private byte vx, vy;
        @Override
        public void run() {
            calendar = Calendar.getInstance();
            curDate = calendar.getTime();
            m = calendar.get(Calendar.MINUTE);

            runOnUiThread(new Runnable() {
                // Отображаем информацию в текстовом поле count:
                @Override
                public void run() {
                    //Log.d(TAG, "TIME.....................=" + s);
                    if (sendTime) {
                            h = calendar.get(Calendar.HOUR);
                            if (analogTime) {
                                sendBlockUpdCmd();
                                sendEraseDisplayCmd();
                                // Циферблат
                                sendDrawRectCmd(8, 0, 48, 48, false);
                                sendDrawRectCmd(52, 22, 3, 3, true);
                                sendDrawRectCmd(30, 44, 3, 3, true);
                                sendDrawRectCmd(9, 22, 3, 3, true);
                                sendDrawRectCmd(30, 1, 3, 3, true);
                                sendDrawDotCmd(42, 4);
                                sendDrawDotCmd(50, 12);
                                sendDrawDotCmd(50, 34);
                                sendDrawDotCmd(42, 42);
                                sendDrawDotCmd(20, 42);
                                sendDrawDotCmd(12, 34);
                                sendDrawDotCmd(12, 12);
                                sendDrawDotCmd(20, 4);
                                // Стрелка минутная
                                vx = Constants.clock_hand_dx[m];
                                vy = Constants.clock_hand_dy[m];
                                sendDrawLineCmd(31, 23, 31 + vx, 23 + vy);
                                // Стрелка часовая
                                if (h < 12)
                                {
                                    vx = Constants.clock_hand_dx[h * 5 + m / 12];
                                    vy = Constants.clock_hand_dy[h * 5 + m / 12];
                                } else
                                {
                                    vx = Constants.clock_hand_dx[(h - 12) * 5 + m / 12];
                                    vy = Constants.clock_hand_dy[(h - 12) * 5 + m / 12];
                                }
                                sendDrawLineCmd(31, 23, 31 + vx * 6 / 8, 23 + vy * 6 / 8);
                                // Секунды
                                sendDrawLineCmd(0, 47, 0, 48 - calendar.get(Calendar.SECOND) * 48 / 59);
                                sendDrawLineCmd(63, 0, 63, calendar.get(Calendar.SECOND) * 48 / 59);
                                sendUnblockUpdCmd();
                            } else {
                                sendBlockUpdCmd();
                                sendEraseDisplayCmd();
                                tmp.setLength(0);

                                sendSetCursorPosCmd(3, 0);
                                sendTextSizeCmd(2);
                                tmp.append(sdf_time.format(curDate));
                                sendDrawStrCmd(tmp.toString());
                                tmp.setLength(0);
                                delay(18); // дать время на отрисовку увеличенным шрифтом

                                sendSetCursorPosCmd(26, 15);
                                sendTextSizeCmd(1);
                                tmp.append(sdf_secs.format(curDate));
                                sendDrawStrCmd(tmp.toString());
                                //etTextForSend.setText(tmp.toString());
                                tmp.setLength(0);

                                tmp.append(sdf_date.format(curDate));
                                tmp.append(' ');
                                tmp.append(sdf_day.format(curDate));
                                sendSetCursorPosCmd(32 - tmp.length() * 3, 36); // tmp.length() д.б. < 11
                                sendTextSizeCmd(1);
                                sendDrawStrCmd(tmp.toString());

                                sendUnblockUpdCmd();
                            }
                    }
                }});
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Bluetooth адаптер своего телефона
        btAdapter = BluetoothAdapter.getDefaultAdapter();

        // Если btAdapter = null - Bluetooth не поддерживается, завершить приложение
        if (btAdapter == null) {
            Toast.makeText(this, R.string.bt_not_support, Toast.LENGTH_LONG).show();
            finish();
        }

        // Инициализировать переменные пользовательского интерфейса
        btnRun = (Button) findViewById(R.id.btnRun);
        etTextForSend = (EditText) findViewById(R.id.etTextForSend);
        etCurPosX = (EditText) findViewById(R.id.etCurPosX);
        etCurPosY = (EditText) findViewById(R.id.etCurPosY);
        etTextSize = (EditText) findViewById(R.id.etTextSize);
        chbWrap = (CheckBox) findViewById(R.id.chbWrap);
        rgTextColor = (RadioGroup) findViewById(R.id.rgTextColor);
        rgColorMode = (RadioGroup) findViewById(R.id.rgColorMode);
        etX1 = (EditText) findViewById(R.id.etX1);
        etY1 = (EditText) findViewById(R.id.etY1);
        etX2 = (EditText) findViewById(R.id.etX2);
        etY2 = (EditText) findViewById(R.id.etY2);
        etContrast = (EditText) findViewById(R.id.etContrast);
        tvTextColor = (TextView) findViewById(R.id.tvTextColor);
        chbInvertDisplay = (CheckBox) findViewById(R.id.chbInvertDisplay);
        tvDrawModeCap = (TextView) findViewById(R.id.tvDrawModeCap);
        tvX2 = (TextView) findViewById(R.id.tvX2);
        tvY2 = (TextView) findViewById(R.id.tvY2);
        tvX1 = (TextView) findViewById(R.id.tvX1);
        tvY1 = (TextView) findViewById(R.id.tvY1);
        chbFillMode = (CheckBox) findViewById(R.id.chbFillMode);
        showDrawMode();
        etRotate = (EditText) findViewById(R.id.etRotate);

        etImgX = (EditText) findViewById(R.id.etImgX);
        etImgY = (EditText) findViewById(R.id.etImgY);
        ivImage = (ImageView) findViewById(R.id.ivImage);
        imgSender = new ImageSender(this);
        imgSender.registerCallBack(this);
        imgNames = getResources().getStringArray(R.array.image_list);
        imageID = getResources().getIdentifier(imgNames[imageIndex], "drawable", getPackageName());

        secTimer = new Timer();
        secTask = new secTimerTask();
        secTimer.schedule(secTask, 0, 1000);

        // Зарегистрировать BroadcastReceiver для отслеживания события обнаружения
        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(brReceiver, filter);

        // Зарегистрировать BroadcastReceiver для отслеживания событий окончания обнаружения
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        this.registerReceiver(brReceiver, filter);

        // Зарегистрировать BroadcastReceiver для отслеживания событий включения/выключения Bluetooth
        filter = new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED);
        this.registerReceiver(brReceiver, filter);
    }

    // Хендлер для приема и обработки сообщений
    public Handler getHandler() {
        return new BtHandler(this);
    }

    // Handler приема информации от службы Bluetooth
    private static class BtHandler extends Handler {
        //Using a weak reference means you won't prevent garbage collection
        private final WeakReference<MainActivity> maWeakReference;

        public BtHandler(MainActivity maInstance) {
            maWeakReference = new WeakReference<MainActivity>(maInstance);
        }

        @Override
        public void handleMessage(android.os.Message msg) {
            MainActivity ma = maWeakReference.get();
            if (ma != null) {
                switch (msg.what) {
                    case Constants.MESSAGE_STATE_CHANGE:
                        switch (msg.arg1) {
                            case BluetoothService.STATE_CONNECTED:
                                ma.btnRun.setEnabled(true);
                                ma.btConnected = true;
                                ma.setStatus(ma.getString(R.string.title_connected_to, ma.deviceName));
                                break;
                            case BluetoothService.STATE_CONNECTING:
                                ma.setStatus(R.string.title_connecting);
                                break;
                            case BluetoothService.STATE_LISTEN:
                            case BluetoothService.STATE_NONE:
                                ma.btConnected = false;
                                ma.sendTime = false;
                                ma.btnRun.setEnabled(false);
                                ma.setStatus(R.string.title_not_connected);
                                break;
                        }
                        break;
                    case Constants.MESSAGE_WRITE:
                        break;
                    case Constants.MESSAGE_READ:
                        break;
                    case Constants.MESSAGE_DEVICE_NAME:
                        // Запомнить имя устройства
                        ma.deviceName = msg.getData().getString(Constants.DEVICE_NAME);
                        Toast.makeText(ma, ma.getString(R.string.msg_connected) + ma.deviceName, Toast.LENGTH_SHORT).show();
                        break;
                    case Constants.MESSAGE_TOAST:
                        Toast.makeText(ma, msg.getData().getString(Constants.TOAST), Toast.LENGTH_SHORT).show();
                        break;
                }
            }
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        // Если Bluetooth не включен - запросить включение, тогда setupConnection() будет
        // вызван в процессе onActivityResult
        if (!btAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
            // Если Bluetooth включен - настроить Bluetooth-соединения
        } else if (btService == null) {
            setupConnection();
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Остановить службу Bluetooth
        if (btService != null) {
            btService.stop();
        }
        // Остановить обнаружение
        if (btAdapter != null) {
            btAdapter.cancelDiscovery();
        }
        // Дерегистрировать BroadcastReceiver
        this.unregisterReceiver(brReceiver);
    }

    @Override
    public void onResume() {
        super.onResume();
        // Запуск службы Bluetooth
        if (btService != null) {
            // Только если состояние STATE_NONE - служба не запущена, запустить
            if (btService.getState() == BluetoothService.STATE_NONE) {
                btService.start();
            }
        }
    }

    @Override
    public void sendPartialImage() { // коллбек-функция отправки очередной порции картинки
        sendImage(imagePacket);
    }

    private void delay(int millis) {
        try
        {
            Thread.sleep(millis);
        }
        catch(InterruptedException ex)
        {
            Thread.currentThread().interrupt();
        }
    }

    private int strToInt(String s, int Default) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {
            return Default;
        }
    }

    private void sendInt(int value, char cmd) {
        StringBuilder str = new StringBuilder();

        str.delete(0, str.length());
        str.append('#');
        str.append(Constants.HEX.charAt(2 + 1)); // + 1, т.е. учесть символ команды cmd
        str.append(cmd);
        str.append(String.format(getString(R.string.hex_format), value));
        sendMessage(str.toString());
    }

    private void sendTwoInt(int value1, int value2, char cmd) {
        StringBuilder str = new StringBuilder();

        str.delete(0, str.length());
        str.append('#');
        str.append(Constants.HEX.charAt(4 + 1)); // + 1, т.е. учесть символ команды cmd
        str.append(cmd);
        str.append(String.format(getString(R.string.hex_format), value1));
        str.append(String.format(getString(R.string.hex_format), value2));
        sendMessage(str.toString());
    }

    public void sendBlockUpdCmd() {
        sendMessage("#1B");
    }

    public void sendUnblockUpdCmd() {
        sendMessage("#1U");
    }

    public void sendEraseDisplayCmd() {
        sendMessage("#1E");
    }

    public void sendFillDisplayCmd() {
        sendMessage("#1F");
    }

    public void sendSetCursorPosCmd(int x, int y) {
        sendTwoInt(x, y, 'P');
    }

    public void sendTextSizeCmd(int x) {
        sendInt(x, 'X');
    }

    public void sendDrawDotCmd(int x, int y) {
        sendTwoInt(x, y, 'D');
    }

    public void sendDrawStrCmd(String value) {
        StringBuilder str = new StringBuilder();
        int n;

        n = value.length();
        if (n > 0) {
            str.append('#');
            str.append(Constants.HEX.charAt(n + 1)); // + 1, т.е. учесть символ команды 'S'
            str.append('S');
            str.append(value);
            sendMessage(str.toString());
        }
    }

    public void sendDrawLineCmd(int x1, int y1, int x2, int y2) {
        StringBuilder str = new StringBuilder();

        str.append('#');
        str.append(Constants.HEX.charAt(8 + 1)); // + 1, т.е. учесть символ команды 'L'
        str.append('L');
        str.append(String.format(getString(R.string.hex_format), x1));
        str.append(String.format(getString(R.string.hex_format), y1));
        str.append(String.format(getString(R.string.hex_format), x2));
        str.append(String.format(getString(R.string.hex_format), y2));
        sendMessage(str.toString());
    }

    public void sendDrawRectCmd(int x, int y, int w, int h, boolean fill) {
        StringBuilder str = new StringBuilder();

        str.append('#');
        str.append(Constants.HEX.charAt(8 + 1)); // + 1, т.е. учесть символ команды
        if (fill) {
            str.append('N');
        } else {
            str.append('R');
        }
        str.append(String.format(getString(R.string.hex_format), x));
        str.append(String.format(getString(R.string.hex_format), y));
        str.append(String.format(getString(R.string.hex_format), w));
        str.append(String.format(getString(R.string.hex_format), h));
        sendMessage(str.toString());
    }

    public void sendDrawCircleCmd(int x, int y, int r, boolean fill) {
        StringBuilder str = new StringBuilder();

        str.delete(0, str.length());
        str.append('#');
        str.append(Constants.HEX.charAt(6 + 1)); // + 1, т.е. учесть символ команды
        if (fill) {
            str.append('H');
        } else {
            str.append('C');
        }
        str.append(String.format(getString(R.string.hex_format), x));
        str.append(String.format(getString(R.string.hex_format), y));
        str.append(String.format(getString(R.string.hex_format), r));
        sendMessage(str.toString());
    }

    public void btnRunClick(View v) {
        AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
        builder.setTitle(R.string.lbl_choise_cmd);
        builder.setAdapter(new HighlightAdapter(this, getResources().getStringArray(R.array.action_list)), new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
                if (btConnected) {
                    if (item != Constants.DRAW_CLOCK) {
                        sendTime = false;
                    }
                    doAction(item);
                }
            }
        });
        builder.setIcon(R.drawable.ic_launcher);
        builder.setCancelable(true);
        builder.setNegativeButton(R.string.lbl_cancel_cmd,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        dialog.cancel();
                    }
                });
        AlertDialog alert = builder.create();
        alert.show();
    }

    public void showDrawMode() {
        switch (drawMode) {
            case 0: // draw dot
                tvDrawModeCap.setText(R.string.lbl_dot_mode_cap);
                chbFillMode.setEnabled(false);

                etY2.setVisibility(View.INVISIBLE);
                tvY2.setVisibility(View.INVISIBLE);
                etX2.setVisibility(View.INVISIBLE);
                tvX2.setVisibility(View.INVISIBLE);

                tvY1.setText(R.string.lbl_Y0_cap);
                tvX1.setText(R.string.lbl_X0_cap);
                break;
            case 1: // draw line
                tvDrawModeCap.setText(R.string.lbl_line_mode_cap);
                chbFillMode.setEnabled(false);
                etY2.setVisibility(View.VISIBLE);
                etY2.setHint(R.string.lbl_0_47);

                tvY2.setVisibility(View.VISIBLE);
                tvY2.setText(R.string.lbl_Y2_cap);

                etX2.setVisibility(View.VISIBLE);
                etX2.setHint(R.string.lbl_0_63);

                tvX2.setVisibility(View.VISIBLE);
                tvX2.setText(R.string.lbl_X2_cap);

                tvY1.setText(R.string.lbl_Y1_cap);
                tvX1.setText(R.string.lbl_X1_cap);
                break;
            case 2: // draw rect
                tvDrawModeCap.setText(R.string.lbl_rect_mode_cap);
                chbFillMode.setEnabled(true);
                etY2.setVisibility(View.VISIBLE);
                etY2.setHint(R.string.lbl_0_48);

                tvY2.setVisibility(View.VISIBLE);
                tvY2.setText(R.string.lbl_rectH_cap);

                etX2.setVisibility(View.VISIBLE);
                etX2.setHint(R.string.lbl_0_64);

                tvX2.setVisibility(View.VISIBLE);
                tvX2.setText(R.string.lbl_rectW_cap);

                tvY1.setText(R.string.lbl_Y0_cap);
                tvX1.setText(R.string.lbl_X0_cap);
                break;
            case 3: // draw circle
                tvDrawModeCap.setText(R.string.lbl_circle_mode_cap);
                chbFillMode.setEnabled(true);
                etY2.setVisibility(View.INVISIBLE);
                tvY2.setVisibility(View.INVISIBLE);

                etX2.setVisibility(View.VISIBLE);
                etX2.setHint(R.string.lbl_0_63);

                tvX2.setVisibility(View.VISIBLE);
                tvX2.setText(R.string.lbl_circleR_cap);

                tvY1.setText(R.string.lbl_Y0_cap);
                tvX1.setText(R.string.lbl_X0_cap);
                break;
        }
    }

    public void btnSelFigureClick(View v) {
        drawMode++;
        if (drawMode > 3) { // 0-Dot, 1-Line, 2-Rect, 3-Circle
            drawMode = 0;
        }
        showDrawMode();
    }

    private void showTextColorMode() {
        if (textColorMode) {
            tvTextColor.setText(R.string.lbl_text_color_cap);
            ((RadioButton) rgTextColor.getChildAt(textColor)).setChecked(true);
        } else {
            tvTextColor.setText(R.string.lbl_text_bgcolor_cap);
            ((RadioButton) rgTextColor.getChildAt(textBgColor)).setChecked(true);
        }
    }

    public void btnSelTextColorClick(View v) {
        if (textColorMode) {
            RadioButton rbTextColor = (RadioButton) findViewById(rgTextColor.getCheckedRadioButtonId());
            textColor = rgTextColor.indexOfChild(rbTextColor);
        } else {
            RadioButton rbTextColor = (RadioButton) findViewById(rgTextColor.getCheckedRadioButtonId());
            textBgColor = rgTextColor.indexOfChild(rbTextColor);
        }
        textColorMode = !textColorMode;
        showTextColorMode();
    }

    public void btnSelImageClick(View v) {
        imageIndex++;
        if (imageIndex >= imgNames.length) {
            imageIndex = 0;
        }
        imageID = getResources().getIdentifier(imgNames[imageIndex], "drawable", getPackageName());
        ivImage.setImageResource(imageID);
    }

    private void doAction(int actionIndex) {
        int n, x, y, x1, y1;

        switch (actionIndex) {
            case Constants.CLEAR_DISPLAY:
                sendTwoInt(0, 0, 'P');
                sendEraseDisplayCmd();
                break;
            case Constants.FILL_DISPLAY:
                sendTwoInt(0, 0, 'P');
                sendFillDisplayCmd();
                break;
            case Constants.INVERT_DISPLAY:
                if (chbInvertDisplay.isChecked())
                    sendMessage("#3I01");
                else
                    sendMessage("#3I00");
                break;
            case Constants.SET_CUR_POS:
                x = strToInt(etCurPosX.getText().toString(), Constants.LCD_WIDTH);
                y = strToInt(etCurPosY.getText().toString(), Constants.LCD_HEIGHT);
                if (x > (Constants.LCD_WIDTH - 1)) x = 0; // ограничение длины и ввод только цифр реализовано в xml;
                if (y > (Constants.LCD_HEIGHT - 1)) y = 0; // здесь только ограничение на максимум
                etCurPosX.setText(Integer.toString(x));
                etCurPosY.setText(Integer.toString(y));
                sendSetCursorPosCmd(x, y);
                break;
            case Constants.SET_SIZE:
                x = strToInt(etTextSize.getText().toString(), 1);
                sendTextSizeCmd(x);
                break;
            case Constants.SET_WRAP:
                if (chbWrap.isChecked()) {
                    sendInt(1, 'W');
                } else {
                    sendInt(0, 'W');
                }
                break;
            case Constants.SET_TXT_COLOR:
                sendTwoInt(textColor, textBgColor, 'T');
                break;
            case Constants.SET_COLOR:
                RadioButton rbColor = (RadioButton) findViewById(rgColorMode.getCheckedRadioButtonId());
                x = rgColorMode.indexOfChild(rbColor);
                sendInt(x, 'Y');
                break;
            case Constants.DRAW_STRING:
                if (etTextForSend.getText() != null) {
                    sendDrawStrCmd(etTextForSend.getText().toString());
                }
                break;
            case Constants.DRAW_DOT:
                x = strToInt(etX1.getText().toString(), Constants.LCD_WIDTH);
                y = strToInt(etY1.getText().toString(), Constants.LCD_HEIGHT);
                if (x > (Constants.LCD_WIDTH - 1)) x = 0; // ограничение длины и ввод только цифр реализовано в xml;
                if (y > (Constants.LCD_HEIGHT - 1)) y = 0; // здесь только ограничение на максимум
                etX1.setText(Integer.toString(x));
                etY1.setText(Integer.toString(y));
                sendDrawDotCmd(x, y);
                break;
            case Constants.DRAW_LINE:
                x = strToInt(etX1.getText().toString(), Constants.LCD_WIDTH);
                y = strToInt(etY1.getText().toString(), Constants.LCD_HEIGHT);
                x1 = strToInt(etX2.getText().toString(), Constants.LCD_WIDTH);
                y1 = strToInt(etY2.getText().toString(), Constants.LCD_HEIGHT);
                if (x > (Constants.LCD_WIDTH - 1)) x = 0; // ограничение длины и ввод только цифр реализовано в xml;
                if (y > (Constants.LCD_HEIGHT - 1)) y = 0; // здесь только ограничение на максимум
                if (x1 > (Constants.LCD_WIDTH - 1)) x1 = 0;
                if (y1 > (Constants.LCD_HEIGHT - 1)) y1 = 0;
                etX1.setText(Integer.toString(x));
                etY1.setText(Integer.toString(y));
                etX2.setText(Integer.toString(x1));
                etY2.setText(Integer.toString(y1));
                n = (int)Math.sqrt((x1-x)*(x1-x) + (y1-y)*(y1-y));
                if (n == 0)
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_line_len, Toast.LENGTH_SHORT).show();
                } else {
                    sendDrawLineCmd(x, y, x1, y1);
                }
                break;
            case Constants.DRAW_RECT:
                x = strToInt(etX1.getText().toString(), Constants.LCD_WIDTH);
                y = strToInt(etY1.getText().toString(), Constants.LCD_HEIGHT);
                x1 = strToInt(etX2.getText().toString(), Constants.LCD_WIDTH);  // wight
                y1 = strToInt(etY2.getText().toString(), Constants.LCD_HEIGHT); // height
                if (x > (Constants.LCD_WIDTH - 1)) x = 0; // ограничение длины и ввод только цифр реализовано в xml;
                if (y > (Constants.LCD_HEIGHT - 1)) y = 0; // здесь только ограничение на максимум
                if (x1 > Constants.LCD_WIDTH) x1 = 0;
                if (y1 > Constants.LCD_HEIGHT) y1 = 0;
                etX1.setText(Integer.toString(x));
                etY1.setText(Integer.toString(y));
                etX2.setText(Integer.toString(x1));
                etY2.setText(Integer.toString(y1));
                if (x1 == 0)
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_rect_width, Toast.LENGTH_SHORT).show();
                    return;
                }
                if (y1 == 0)
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_rect_height, Toast.LENGTH_SHORT).show();
                    return;
                }
                if (((x + x1) > Constants.LCD_WIDTH) || ((y + y1) > Constants.LCD_HEIGHT))
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_rect_bounds, Toast.LENGTH_SHORT).show();
                    return;
                }
                sendDrawRectCmd(x, y, x1, y1, chbFillMode.isChecked());
                break;
            case Constants.DRAW_CIRCLE:
                x = strToInt(etX1.getText().toString(), Constants.LCD_WIDTH);
                y = strToInt(etY1.getText().toString(), Constants.LCD_HEIGHT);
                x1 = strToInt(etX2.getText().toString(), Constants.LCD_WIDTH); // radius

                if (x > (Constants.LCD_WIDTH - 1)) x = 0; // ограничение длины и ввод только цифр реализовано в xml;
                if (y > (Constants.LCD_HEIGHT - 1)) y = 0; // здесь только ограничение на максимум
                if (x1 > (Constants.LCD_WIDTH - 1)) x1 = 0;
                etX1.setText(Integer.toString(x));
                etY1.setText(Integer.toString(y));
                etX2.setText(Integer.toString(x1));

                if (x1 == 0)
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_circle_radius, Toast.LENGTH_SHORT).show();
                    return;
                }
                if (((x + x1) > Constants.LCD_WIDTH) || ((y + x1) > Constants.LCD_HEIGHT))
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_circle_bounds, Toast.LENGTH_SHORT).show();
                    return;
                }
                if (((x - x1) < 0) || ((y - x1) < 0))
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_circle_bounds, Toast.LENGTH_SHORT).show();
                    return;
                }
                sendDrawCircleCmd(x, y, x1, chbFillMode.isChecked());
                break;
            case Constants.SET_CONTRAST:
                x = strToInt(etContrast.getText().toString(), 0);
                if (x > 255) x = 255; // ограничение длины и ввод только цифр реализовано в xml;
                if (x < 1) x = 1; // здесь только ограничение на максимум/минимум
                etContrast.setText(Integer.toString(x));
                sendInt(x, 'A');
                break;
            case Constants.SET_ROTATE:
                x = strToInt(etRotate.getText().toString(), 0);
                if (x > 3) x = 3; // ограничение длины и ввод только цифр реализовано в xml;
                etRotate.setText(Integer.toString(x));
                sendInt(x, 'O');
                break;
            case Constants.DRAW_MSG:
                if (etTextForSend.getText() != null) {
                    sendBlockUpdCmd();
                    sendTwoInt(0, 0, 'P'); // set cursor position
                    sendEraseDisplayCmd();
                    // draw string
                    sendDrawStrCmd(etTextForSend.getText().toString());
                    sendUnblockUpdCmd();
                }
                break;
            case Constants.DRAW_CLOCK:
                analogTime = false;
                sendTime = !sendTime;
                break;
            case Constants.DRAW_AN_CLOCK:
                analogTime = true;
                sendTime = !sendTime;
                break;
            case Constants.DRAW_IMAGE:
                x = strToInt(etImgX.getText().toString(), Constants.LCD_WIDTH);
                y = strToInt(etImgY.getText().toString(), Constants.LCD_HEIGHT);
                if (x > (Constants.LCD_WIDTH - 1)) x = 0; // ограничение длины и ввод только цифр реализовано в xml;
                if (y > (Constants.LCD_HEIGHT - 1)) y = 0; // здесь только ограничение на максимум
                etImgX.setText(Integer.toString(x));
                etImgY.setText(Integer.toString(y));
                if (!imgSender.Prepare(imageID, x, y))
                {
                    Toast.makeText(MainActivity.this, R.string.msg_err_image_bounds, Toast.LENGTH_SHORT).show();
                    return;
                }
                sendBlockUpdCmd();
                // подготовать общую часть пакетов
                imagePacket[0] = '#';
                imagePacket[1] = 'F';
                imagePacket[2] = 'G';
                // разобрать картинку на подкартинки и вывести в коллбек функции
                imgSender.Go(x, y, imagePacket);
                sendUnblockUpdCmd();
                break;
        }
    }

    private void sendImage(byte[] value) {
        // Check that we're actually connected before trying anything
        if ((btService != null) && (btService.getState() != BluetoothService.STATE_CONNECTED)) {
            Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
            return;
        }

        // Check that there's actually something to send
        if (value.length > 0) {
            btService.write(value);
        }
    }

    private void sendMessage(String message) {
        byte[] send = {};
        // Check that we're actually connected before trying anything
        if ((btService != null) && (btService.getState() != BluetoothService.STATE_CONNECTED)) {
            Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
            return;
        }

        // Check that there's actually something to send
        if (message.length() > 0) {
            // Get the message bytes and tell the BluetoothChatService to write
            // Перекодировка в Windows-1251, если возможно
            if (Charset.isSupported(getString(R.string.chs_name_win_1251)))
                try {
                    send = message.getBytes(getString(R.string.chs_name_win_1251));
                } catch (Exception e) {
                    //
                }
            else if (Charset.isSupported(getString(R.string.chs_name_cp_1251)))
                try {
                    send = message.getBytes(getString(R.string.chs_name_cp_1251));
                } catch (Exception e) {
                    //
                }
                 else
                    send = message.getBytes();
            btService.write(send);
        }
    }

    // Настройка Bluetooth-соединения
    private void setupConnection() {
        Log.d(TAG, "setupConnection()");
        // Создание BluetoothService
        btService = new BluetoothService(this, getHandler());
    }

    // Обновить статус на ActionBar
    private void setStatus(int resId) {
        final ActionBar actionBar = getActionBar();
        if (null == actionBar) {
            return;
        }
        actionBar.setSubtitle(resId);
    }

    // Обновить статус на ActionBar
    private void setStatus(CharSequence subTitle) {
        final ActionBar actionBar = getActionBar();
        if (null == actionBar) {
            return;
        }
        actionBar.setSubtitle(subTitle);
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case REQUEST_ENABLE_BT:
                // Юзер подтвердил включение Bluetooth
                if (resultCode == Activity.RESULT_OK) {
                    // setup Bluetooth
                    setupConnection();
                } else {
                    // Юзер не разрешил включение Bluetooth - завершение приложения
                    Log.d(TAG, "BT not enabled");
                    Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                    finish();
                }
        }
    }

    // Установить соединение
    private void connectDevice() {
        BluetoothDevice device = btAdapter.getRemoteDevice(deviceMacAddr);
        // Попытка подключиться
        btService.connect(device, secureConnect);
    }

    // Поиск имени прибора в списке спаренных устройств. Инициализирует deviceMacAddr
    public void findPairedDevice()
    {
        Set<BluetoothDevice> pairedDevices = btAdapter.getBondedDevices();

        deviceIsFound = false;
        if (pairedDevices.size() > 0) // Если список спаренных устройств не пуст
        {
            for (BluetoothDevice device: pairedDevices) // проходимся в цикле по этому списку
            {
                //etTextForSend.setText(device.getName());
                if (Constants.BT_DEVICE_NAME.equals(device.getName()))
                {
                    deviceMacAddr = device.getAddress();
                    deviceIsFound = true;
                    break;
                }
            }
        }
    }

    // Вызов соединения
    private void connectToDevice()
    {
        // Если Bluetooth сейчас включен - поиск/обнаружение и подключение
        if (btAdapter.isEnabled()) {
            // Поиск имени прибора в списке спаренных устройств
            findPairedDevice();
            if (!deviceIsFound) // Прибора нет в списке спаренных - запуск обнаружения
            {
                progressDialog = ProgressDialog.show(this, this.getString(R.string.msg_find), this.getString(R.string.msg_wait));
                // После обнаружения - подключение в brReceiver.onReceive
                btAdapter.startDiscovery();
            } else // Прибор ЕСТЬ в списке спаренных - подключаемся
            {
                connectDevice();
            }
        }
    }

    // События при обнаружении, окончании процесса обнаружения и включении/выключении Bluetooth
    private final BroadcastReceiver brReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {

            String action = intent.getAction();
            if (BluetoothDevice.ACTION_FOUND.equals(action)) {

                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                if (Constants.BT_DEVICE_NAME.equals(device.getName()))
                {
                    deviceMacAddr = device.getAddress();
                    deviceIsFound = true;
                    btAdapter.cancelDiscovery();
                    // Система сама запросит пароль
                    connectDevice();
                }
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
                // По окончании обнаружения скрыть progressDialog
                if (progressDialog != null)
                    progressDialog.dismiss();
            } else if (action.equals(BluetoothAdapter.ACTION_STATE_CHANGED)) {
                // События включения/выключения Bluetooth
                final int state = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, BluetoothAdapter.ERROR);
                switch (state) {
                    case BluetoothAdapter.STATE_OFF:
                        finish(); // Завершение приложения при выключении Bluetooth
                        break;
                    case BluetoothAdapter.STATE_TURNING_OFF:
                        break;
                    case BluetoothAdapter.STATE_ON:
                        break;
                    case BluetoothAdapter.STATE_TURNING_ON:
                        break;
                }
            }
        }
    };

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Создать меню по шаблону "menu_main"
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId()) {
            case R.id.secure_connect_scan: {
                // Пункт меню - "Соединить"
                secureConnect = true;
                connectToDevice();
                return true;
            }
            case R.id.show_app_about: {
                // Пункт меню - "О комплексе "BtDog"
                AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
                builder.setTitle(R.string.app_about_title)
                        .setMessage(Html.fromHtml(getString(R.string.app_about)))
                        .setIcon(R.drawable.ic_launcher)
                        .setCancelable(true)
                        .setNegativeButton("ОК",
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int id) {
                                        dialog.cancel();
                                    }
                                });
                AlertDialog alert = builder.create();
                alert.show();
                TextView txtView = (TextView) alert.findViewById(android.R.id.message);
                txtView.setTextSize(TypedValue.COMPLEX_UNIT_PX, 22);
                return true;
            }
        }
        return false;
    }
}
