/*********************************************************************
Project : BTDOG - Bluetooth Display On Glasses
Version : 0.02 [Click "Project -> Show All Files" before compile!!!]
Date    : 25.04.2017-19.05.2017-27.05.2017
Author  : Vershinin I.V.
Company : My Sweet Home
Comments: BTDOG -       
   ,    .  
     ,     
  Bluetooth-   .

  19.05.2017  .
  27.05.2017   ,  .. / 
  .

Chip type               : ATmega168
AVR Core Clock frequency: 8,000000 MHz
FLASH usage             : 10200 bytes
RAM usage               : 765 bytes
EEPROM usage            : 0 bytes
Main Board              : Arduino Pro Mini (RobotDyn)
Display                 : SSD1306 OLED (64x48 pixels), I2C
Keyboard                : One button, on pin D3

~~~~~~~~~~~~~~~~~~~~~~~~   / DEPENDENCIES  ~~~~~~~~~~~~~~~~~~~~~~~~

  Using modified Arduino core libs:
    "HardwareSerial.h" - for reduction serial buffers size to 32 instead of 64;
    "twi.h" - for reduction TWI_BUFFER_LENGTH to 24 instead of 32;
    "Wire.h" - for reduction BUFFER_LENGTH to 24 instead of 32.

  Using modified Adafruit libs:
    "Adafruit_GFX" - reduced and renamed to "Adafruit_GFX_Reduced";
    "Adafruit_SSD1306" - reduced and renamed to "Adafruit_SSD1306_Reduced".
*********************************************************************/

#include <stdbool.h>
#include <string.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "libs/Adafruit_SSD1306_Reduced/Adafruit_GFX_Reduced.h"
#include "libs/Adafruit_SSD1306_Reduced/Adafruit_SSD1306_Reduced.h"

#define UART_SPEED            38400 // 0.16 % error for F_CPU = 8.0 MHz
#define SSD1306_I2C_ADDR      0x3C
#define REC_BUF_SIZE          16    //   
#define REC_PACKET_START_CHAR '#'   //   
#define REC_TIMEOUT           10    //  , 

#define BTN_INT_PIN           INTF1  //    (Arduino.D3; ATmega168.INT1)

Adafruit_SSD1306_Reduced screen;

//  
enum recModes
{
  IDLE_MODE,            //   
  GET_PACKET_LEN_MODE,  //    
  REC_PACKET_MODE,      //    
  EXEC_CMD_MODE,        //  
} recMode;

//    
unsigned char recBufIdx;
//   (       )
unsigned char recBuf[REC_BUF_SIZE];
unsigned char recLen;    //  
unsigned long oldMillis; //   
unsigned char errCode;   //  ; 0 -  

//   /
unsigned char curColor = WHITE;

//  
unsigned char flags;
#define bitToggle(value, bit)    ((value) ^= (1UL << (bit)))
#define UPDATE_DISPLAY_FLAG_POS  0
#define UPDATE_DISPLAY_UNBLOCK   bitSet(flags, UPDATE_DISPLAY_FLAG_POS)
#define UPDATE_DISPLAY_BLOCK     bitClear(flags, UPDATE_DISPLAY_FLAG_POS)
#define UPDATE_DISPLAY_IS_SET    bitRead(flags, UPDATE_DISPLAY_FLAG_POS)

#define NEED_UPDATE_FLAG_POS     1
#define NEED_UPDATE_SET          bitSet(flags, NEED_UPDATE_FLAG_POS)
#define NEED_UPDATE_CLEAR        bitClear(flags, NEED_UPDATE_FLAG_POS)
#define NEED_UPDATE_IS_SET       bitRead(flags, NEED_UPDATE_FLAG_POS)

#define BT_SWITH_FLAG_POS        2
#define BT_SWITH_SET             bitSet(flags, BT_SWITH_FLAG_POS)
#define BT_SWITH_CLEAR           bitClear(flags, BT_SWITH_FLAG_POS)
#define BT_SWITH_IS_SET          bitRead(flags, BT_SWITH_FLAG_POS)

#define BT_STATE_FLAG_POS        3
#define BT_STATE_IS_SET          bitRead(flags, BT_STATE_FLAG_POS)
#define BT_STATE_TOGGLE          bitToggle(flags, BT_STATE_FLAG_POS)

//  /  BT-
#define BT_POWER_MASK            B00111111
#define BT_POWER_INIT            DDRB |= (BT_POWER_MASK)
#define BT_POWER_ON              PORTB |= (BT_POWER_MASK)
#define BT_POWER_OFF             PORTB &= ~(BT_POWER_MASK)

// ,   / BT-
const char * const bt_messages[] PROGMEM = {"BT\noff", "BT\n on"};

//    (  BT-)
unsigned char x = screen.width() / 2;  // . . 
unsigned char y = screen.height() / 2; // . . 
unsigned char r = 5; //  
signed char vx = 1;  //   X
signed char vy = 2;  //   Y
bool xd, yd = false; // 
        
//   16-  value  10-  (0..15)
unsigned char _hex2uchar(unsigned char value)
{
  unsigned char result = 0;

  if ((value > 64) && (value < 71)) // A..F
  {
    result = value - 55;
  } else
  if ((value > 47) && (value < 58)) // 0..9
  {
    result = value - 48;
  } else
  {
    errCode = 1;
    return 0;
  };
  return result;
}

//  16-    recBuf  10- .
// index -   ; (index + 1) -   
unsigned char hex2uchar(unsigned char index)
{
  unsigned char result = 0;
  unsigned char temp;
  
  errCode = 0;
  result = _hex2uchar(recBuf[index]);
  if (errCode) return 0;

  temp = _hex2uchar(recBuf[index + 1]);
  if (errCode) return 0;

  result = result * 16 + temp;
  return result;
}

//    recBuf
void exec_command()
{
  unsigned char x, y, w, h;
  
  errCode = 2;
  NEED_UPDATE_CLEAR;
  switch (recBuf[0])
  {
    case 'E': // clear display
      screen.clearDisplay();
      NEED_UPDATE_SET;
    break;
    case 'F': // fill display
      screen.fillScreen(curColor);
      NEED_UPDATE_SET;
    break;   
    case 'B': // block update
      UPDATE_DISPLAY_BLOCK;
      NEED_UPDATE_CLEAR;
    break;
    case 'U': // unblock update
      UPDATE_DISPLAY_UNBLOCK;
      NEED_UPDATE_SET;
    break;
    case 'I': // invert display
      if (recLen != 3) return;
      x = hex2uchar(1);  if (errCode) return;
      if (x) screen.invertDisplay(true); else screen.invertDisplay(false);
    break;
    case 'A': // set contrast
      if (recLen != 3) return;
      x = hex2uchar(1);  if (errCode) return;
      screen.ssd1306_command(SSD1306_SETCONTRAST);
      screen.ssd1306_command(x);
    break;
    case 'O': // set rotation
      if (recLen != 3) return;
      x = hex2uchar(1);  if (errCode) return;
      screen.setRotation(x);
    break;
    case 'P': // set cursor pos
      if (recLen != 5) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      screen.setCursor(x, y);
    break;
    case 'L': // draw line
      if (recLen != 9) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      w = hex2uchar(5);  if (errCode) return;
      h = hex2uchar(7);  if (errCode) return;
      screen.drawLine(x, y, w, h, curColor);
      NEED_UPDATE_SET;
    break;
    case 'D': // draw dot (pixel)
      if (recLen != 5) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      screen.drawPixel(x, y, curColor);
      NEED_UPDATE_SET;
    break;
    case 'R': // draw rectangle
      if (recLen != 9) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      w = hex2uchar(5);  if (errCode) return;
      h = hex2uchar(7);  if (errCode) return;
      screen.drawRect(x, y, w, h, curColor);
      NEED_UPDATE_SET;
    break;
    case 'N': // draw filled rectangle
      if (recLen != 9) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      w = hex2uchar(5);  if (errCode) return;
      h = hex2uchar(7);  if (errCode) return;
      screen.fillRect(x, y, w, h, curColor);
      NEED_UPDATE_SET;
    break;
    case 'C': // draw circle
      if (recLen != 7) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      h = hex2uchar(5);  if (errCode) return;
      screen.drawCircle(x, y, h, curColor);
      NEED_UPDATE_SET;
    break;
    case 'H': // draw filled wheel (circle)
      if (recLen != 7) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      h = hex2uchar(5);  if (errCode) return;
      screen.fillCircle(x, y, h, curColor);
      NEED_UPDATE_SET;
    break;
    case 'S': // draw string
      screen.print((char *)(&recBuf[1]));
      NEED_UPDATE_SET;
    break;
    case 'G': // draw image (bitmap)
      if (recLen != 15) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      w = _hex2uchar(recBuf[5]);  if (errCode) return;
      h = _hex2uchar(recBuf[6]);  if (errCode) return;
      screen.drawBitmap(x, y, (unsigned char *)(&recBuf[7]), w, h, curColor);
      NEED_UPDATE_SET;
    break;
    case 'T': // set text color
      if (recLen != 5) return;
      x = hex2uchar(1);  if (errCode) return;
      y = hex2uchar(3);  if (errCode) return;
      screen.setTextColor(x, y);
    break;
    case 'W': // set text wrap
      if (recLen != 3) return;
      x = hex2uchar(1);  if (errCode) return;
      if (x) screen.setTextWrap(true); else screen.setTextWrap(false);
    break;
    case 'X': // set text size
      if (recLen != 3) return;
      x = hex2uchar(1);  if (errCode) return;
      screen.setTextSize(x);
    break;
    case 'Y': // set line dye (color)
      if (recLen != 3) return;
      x = hex2uchar(1);  if (errCode) return;
      if (x > INVERSE) x = WHITE;
      curColor = x;
    break;
  };
  if (UPDATE_DISPLAY_IS_SET && NEED_UPDATE_IS_SET)
  {
    screen.display();
  };
  errCode = 0;
}

//       
void receive_packet()
{
  unsigned char temp;
  
  if (Serial.available() > 0)
  {
    temp = Serial.read();
    //    
    if ((recMode == IDLE_MODE) && (temp == REC_PACKET_START_CHAR))
    {
      recMode = GET_PACKET_LEN_MODE; //     
      oldMillis = millis();          //  
      return;
    };
    //   
    if (recMode == GET_PACKET_LEN_MODE)
    {
      errCode = 0;
      recLen = _hex2uchar(temp);
      if (errCode)               //    =>  
      {
        recMode = IDLE_MODE;
        return;
      };
      recMode = REC_PACKET_MODE; //   =>    
      return;
    };
    //   
    if (recMode == REC_PACKET_MODE)
    {
      recBuf[recBufIdx] = temp;
      recBufIdx++;
      if (recBufIdx == recLen) //   .  =>  
      {
        recBufIdx = 0;
        recMode = EXEC_CMD_MODE; //   
        exec_command();          //  
        memset(recBuf, 0, REC_BUF_SIZE); //   
        recMode = IDLE_MODE;     //   =>   
      };
    };
  };
  //       =>  
  if (((recMode == GET_PACKET_LEN_MODE) || (recMode == REC_PACKET_MODE)) && (millis() - oldMillis > REC_TIMEOUT))
  {
    recBufIdx = 0;
    memset(recBuf, 0, REC_BUF_SIZE);
    recMode = IDLE_MODE;
  };
}

void btn_handler()
{
  BT_SWITH_SET;
}

void bt_switch()
{
  char * bt_msg_ptr;
  unsigned char old_textsize, old_textcolor, old_textbgcolor, old_cursor_x, old_cursor_y;
  
  BT_STATE_TOGGLE;
    
  // switch UART
  if (BT_STATE_IS_SET)
  {
    recMode = IDLE_MODE;
    recBufIdx = 0;
    x = y = 0;
    memset(recBuf, 0, REC_BUF_SIZE);
    Serial.begin(UART_SPEED);
    BT_POWER_ON;
    UPDATE_DISPLAY_UNBLOCK;
  } else
  {
    Serial.end();
    BT_POWER_OFF;
  };
  
  // store screen settings
  old_textsize = screen.getTextSize();
  old_textcolor = screen.getTextColor();
  old_textbgcolor = screen.getTextBgColor();
  old_cursor_x = screen.getCursorX();
  old_cursor_y = screen.getCursorY();  
  // show Bluetooth state
  bt_msg_ptr = (char *) pgm_read_word(&bt_messages[BT_STATE_IS_SET]);
  screen.setTextSize(2);
  screen.setTextColor(1, 1);
  screen.setCursor(12, 8);
  screen.clearDisplay();
  screen.print(bt_msg_ptr);
  screen.display();
  _delay_ms(2000);
  // restore screen settings
  screen.setTextColor(old_textcolor, old_textbgcolor);
  screen.setTextSize(old_textsize);
  screen.setCursor(old_cursor_x, old_cursor_y);
}

void setup()
{
  Serial.end();   //  UART
  ADCSRA = 0;     //  ADC
  BT_POWER_INIT;  //      BT-
  BT_POWER_OFF;   //   BT-
  _delay_ms(200);
  //  ,  
  screen.begin(SSD1306_I2C_ADDR);
  screen.ssd1306_command(0xA0); // /
  screen.ssd1306_command(0xC0); // /
  screen.display();
  _delay_ms(2000);
  //   
  flags = 0;
  //        HIGH  LOW 
  attachInterrupt(BTN_INT_PIN, btn_handler, FALLING);
}
  
void loop()
{
  static unsigned long previousMillis = 0;
  
  if (BT_SWITH_IS_SET)
  {
    bt_switch();
    BT_SWITH_CLEAR;
  };
  
  if (BT_STATE_IS_SET) // BT-  =>  UART
  {
    receive_packet();
  } else             // BT-  =>  
  {
    if (millis() - previousMillis > 100)
    {
      previousMillis = millis();
      //       ,      
      if ((x > (screen.width() - r - 2)) || (x < (1 + r)))
      {
        if (xd == false)
          xd = true;
        else
          xd = false;
      } else
        xd = false;
      //       ,      
      if ((y > (screen.height() - r - 2)) || (y < (2 + r)))
      {
        if (yd == false)
          yd = true;
        else
          yd = false;
      } else
        yd = false;
      if (xd) vx = -vx;
      if (yd) vy = -vy;
      x = x + vx;	//  
      y = y + vy;
      //  
      screen.clearDisplay();
      screen.drawRect(0, 0, screen.width(), screen.height(), WHITE);
      screen.drawCircle(x, y, r, WHITE);
      screen.display();
    };
  };
}