#ifndef _ADAFRUIT_GFX_REDUCED_H
#define _ADAFRUIT_GFX_REDUCED_H

#if ARDUINO >= 100
 #include "Arduino.h"
 #include "Print.h"
#else
 #include "WProgram.h"
#endif

#define BLACK 0
#define WHITE 1
#define INVERSE 2

#ifndef _swap_int8_t
#define _swap_int8_t(a, b) { uint8_t t = a; a = b; b = t; }
#endif

class Adafruit_GFX_Reduced : public Print {

 public:

  Adafruit_GFX_Reduced(int8_t w, int8_t h); // Constructor

  // This MUST be defined by the subclass:
  virtual void drawPixel(uint8_t x, uint8_t y, uint8_t color) = 0;

  // These MAY be overridden by the subclass to provide device-specific
  // optimized code.  Otherwise 'generic' versions are used.
  virtual void
    drawLine(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t color),
    drawFastVLine(uint8_t x, uint8_t y, uint8_t h, uint8_t color),
    drawFastHLine(uint8_t x, uint8_t y, uint8_t w, uint8_t color),
    drawRect(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t color),
    fillRect(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t color),
    fillScreen(uint8_t color),
    invertDisplay(boolean i);

  // These exist only with Adafruit_GFX (no subclass overrides)
  void
    drawCircle(uint8_t x0, uint8_t y0, uint8_t r, uint8_t color),
    drawCircleHelper(uint8_t x0, uint8_t y0, uint8_t r, uint8_t cornername,
      uint8_t color),
    fillCircle(uint8_t x0, uint8_t y0, uint8_t r, uint8_t color),
    fillCircleHelper(uint8_t x0, uint8_t y0, uint8_t r, uint8_t cornername,
      uint8_t delta, uint8_t color),
    drawTriangle(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1,
      uint8_t x2, uint8_t y2, uint8_t color),
    fillTriangle(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1,
      uint8_t x2, uint8_t y2, uint8_t color),
    drawRoundRect(uint8_t x0, uint8_t y0, uint8_t w, uint8_t h,
      uint8_t radius, uint8_t color),
    fillRoundRect(uint8_t x0, uint8_t y0, uint8_t w, uint8_t h,
      uint8_t radius, uint8_t color),
    drawBitmap(uint8_t x, uint8_t y, const uint8_t *bitmap,
      uint8_t w, uint8_t h, uint8_t color),
    drawBitmap(uint8_t x, uint8_t y, const uint8_t *bitmap,
      uint8_t w, uint8_t h, uint8_t color, uint16_t bg),
    drawBitmap(uint8_t x, uint8_t y, uint8_t *bitmap,
      uint8_t w, uint8_t h, uint8_t color),
    drawBitmap(uint8_t x, uint8_t y, uint8_t *bitmap,
      uint8_t w, uint8_t h, uint8_t color, uint16_t bg),
    drawXBitmap(uint8_t x, uint8_t y, const uint8_t *bitmap,
      uint8_t w, uint8_t h, uint8_t color),
    drawChar(uint8_t x, uint8_t y, unsigned char c, uint8_t color,
      uint8_t bg, uint8_t size),
    setCursor(int8_t x, int8_t y),
    setTextColor(uint8_t c),
    setTextColor(uint8_t c, uint8_t bg),
    setTextSize(uint8_t s),
    setTextWrap(boolean w),
    setRotation(uint8_t r);
    
#if ARDUINO >= 100
  virtual size_t write(uint8_t);
#else
  virtual void   write(uint8_t);
#endif

  uint8_t height(void) const;
  uint8_t width(void) const;

  uint8_t getRotation(void) const;

  // get current cursor position (get rotation safe maximum values, using: width() for x, height() for y)
  uint8_t getCursorX(void) const;
  uint8_t getCursorY(void) const;

  uint8_t getTextSize(void) const;
  uint8_t getTextColor(void) const;
  uint8_t getTextBgColor(void) const;
  
 protected:
  const int8_t
    WIDTH, HEIGHT;   // This is the 'raw' display w/h - never changes
  int8_t
    _width, _height, // Display w/h as modified by current rotation
    cursor_x, cursor_y;
  uint8_t
    textcolor, textbgcolor;
  uint8_t
    textsize,
    rotation;
  boolean
    wrap;   // If set, 'wrap' text at right edge of display
};

#endif // _ADAFRUIT_GFX_REDUCED_H
